import fi.jyu.mit.ohj2.*;
/**
 * Ohjelmalla integroidaan numeerisesti funktio-olio.
 * @author Vesa Lappalainen
 * @version 1.0, 25.03.2003
 */
public class Integroi2 {

  interface FunktioRR {
    public double f(double x);
  }

  static class SinFun implements FunktioRR {
    public double f(double x) { return Math.sin(x); }
  }

  static class ExpFun implements FunktioRR {
    public double f(double x) { return Math.exp(x); }
  }

  static class OmaFun implements FunktioRR  {
    public double f(double x) { return 2*x- 5;      }
  }

  static class P2 implements FunktioRR  {
    private double a,b,c;
    public P2(double a, double b, double c) {
      this.a = a; this.b = b; this.c = c;
    }
    public double f(double x) { return a*x*x + b*x + c;  }

    public String toString() {
      return a + "x^2 + " + b + "x + " + c;
    }

  }

  public static double integroi(FunktioRR f, double x1, double x2, int tiheys) {
    double x,dx,summa=0;

    dx = (x2- x1)/tiheys;

    for (x=x1+dx/2 ; x<x2; x+=dx)
      summa += f.f(x)*dx;

    return summa;
  }


  public static void main(String[] args)  {
    double ifx;
    ifx = integroi(new OmaFun(),0,5,100);
    System.out.println("Integraali omafun(x) vlilt [0,5] on noin "+
                       Mjonot.fmt(ifx,7,5));
    ifx = integroi(new SinFun(),0,Math.PI,1000);
    System.out.println("Integraali sin(x) vlilt [0,pi] on noin "+
                       Mjonot.fmt(ifx,7,5));
    ifx = integroi(new ExpFun(),0,1,500);
    System.out.println("Integraali exp(x) vlilt [0,1] on noin "+
                       Mjonot.fmt(ifx,7,5));
    P2 p = new P2(0,-2,1);
    ifx = integroi(p,0,1,500);
    System.out.println("Integraali " + p + " vlilt [0,1] on noin "+
                       Mjonot.fmt(ifx,7,5));
  }
}
