/**
 * Luokka toteuttamaan sovitun julkisen rajapinnan ja Object-
 * luokan metodeja
 * @author  Vesa Lappalainen
 * @version 1.0, 01.02.2003
 */
public class AikaSekF implements AikaRajapinta {

  private AikaF hm = new AikaF();
  private int s;

  /**
   * Asettaa uuden ajan ja pit huolen ett aika on aina oikeaa muotoa.
   * @param h asetettavat tunnit
   * @param m asetettavat minuutit
   * @param s asetettavat sekunnit
   */
  public void aseta(int h,int m, int s) {
    hm.aseta(h,m); this.s = s; lisaa(0,0);
  }

  public void aseta(int h,int m) { aseta(h,m,0); }
  public void aseta(int h)       { aseta(h,0);   }

  public AikaSekF()              { aseta(0,0,0); }

  /**
   * Alustaa ajan
   * @param h tunnit
   * @param m minuutit
   * @param s sekunnit
   */
  public AikaSekF(int h,int m, int s) { // Muodostaja
    aseta(h,m,s);
  }

  public AikaSekF(int h,int m) { aseta(h,m,0); }
  public AikaSekF(int h)       { aseta(h,0,0); }

  /**
   * Tulostaa ajan muodossa 15:05
   * @param lf tulostetaanko rivinvaihto vai ei
   */
  public void tulosta(boolean lf) {
    System.out.print(toString());
    if ( lf ) System.out.println();
  }

  /**
   * Tulostaa ajan muodossa 15:05 sek aina rivinvaihdon
   */
  public void tulosta() {
    tulosta(true);
  }

  /**
   * Lis aikaan valitun minuuttimrn
   * @param lisa_min listtv minuuttimr
   * @param lisa_sek listtv sekunttimr
   */
  public void lisaa(int lisa_min,int lisa_sek)  {
    s += lisa_sek;
    hm.lisaa(lisa_min+s/60);
    s %= 60;
  }

  public void lisaa(int lisa_min)  { lisaa(lisa_min,0); }

  public int getH() { return hm.getH(); }
  public int getM() { return hm.getM(); }
  public int getS() { return s;         }

  public String toString() {
    return hm.toString() + ":" + (s<10?"0":"")+s;
  }

  /**
   * Vertaa kahta kellonaikaa toisiinsa.
   * Nyt on menetetty transitiivisuus, keksi esimerkki.
   * Miten korjataan, vai voiko korjata lainkaan?
   * @param o kellonaika johon verrataan
   * @return ovat ajat samat vai eivt
   */
  public boolean equals(Object o) {
    if ( !(o instanceof AikaSekF) ) return hm.equals(o);
    AikaSekF a = (AikaSekF)o;
    return hm.equals(o) && getS() == a.getS();
  }

  public Object clone() {
    return new AikaSekF(getH(),getM(),getS());
  }

  public int hashCode() {
    return super.hashCode() + getS();
  }

  public static void main(String[] args) throws CloneNotSupportedException {
    AikaF a1 = new AikaF();
    AikaF a2 = new AikaF(13);
    AikaF a3 = new AikaF(14,175);

    a1.tulosta();  a2.tulosta();  a3.tulosta();
    a1.aseta(12,15); a2.aseta(16,-15);
    a1.tulosta(); a2.tulosta();

    AikaSekF a4 = new AikaSekF(14,55,45); a4.tulosta();
    a4.lisaa(3,30); a4.tulosta();

    // Rajapintaan perustuva esimerkki polymorfisesta taulukosta
    AikaRajapinta ajat[] = new AikaRajapinta[8];
    ajat[0] = a1; ajat[1] = a2; ajat[2] = a3; ajat[3] = a4;
    ajat[4] = new AikaSekF(23,59,59);
    ajat[5] = new AikaSekF(12,15,02);
    ajat[6] = new AikaSekF(12,15,03);
    ajat[7] = (AikaSekF)a4.clone();

    System.out.println(ajat[0].equals(ajat[5]));
    System.out.println(ajat[5].equals(ajat[0]));
    System.out.println(ajat[0].equals(ajat[6]));
    System.out.println(ajat[6].equals(ajat[0]));
    System.out.println(ajat[5].equals(ajat[6]));

    for (int i=0; i < ajat.length; i++ ) {
      ajat[i].tulosta(false); System.out.print(" +" + i + " => ");
      ajat[i].lisaa(i); ajat[i].tulosta();
    }
    System.out.println(ajat[4].toString());


  }



}

